@@ -19,44 +19,29 @@
 #include "xfunction.hpp"
 #include "xscalar.hpp"
 #include "xstrides.hpp"
+#include "xstrided_view.hpp"
+#include "xmanipulation.hpp"
 
 namespace xt
 {
 
-    template <class F, class R, class... CT>
-    class xoptional_function;
-
     /***********
      * helpers *
      ***********/
 
 #define UNARY_OPERATOR_FUNCTOR_IMPL(NAME, OP, R)                                \
-    template <class T>                                                          \
     struct NAME                                                                 \
     {                                                                           \
-        template <class U, class RT>                                            \
-        using fvt_t = xt::detail::functor_value_type_t<U, RT>;                  \
-        template <class U, class RT>                                            \
-        using fst_t = xt::detail::functor_batch_simd_type_t<U, RT>;             \
-        using argument_type = T;                                                \
-        using result_type = fvt_t<T, R>;                                        \
-        using simd_value_type = xsimd::simd_type<T>;                            \
-        using simd_result_type = fst_t<simd_value_type, R>;                     \
-        constexpr result_type operator()(const T& arg) const                    \
+        template <class A1>                                                     \
+        constexpr auto operator()(const A1& arg) const                          \
         {                                                                       \
             return OP arg;                                                      \
         }                                                                       \
         template <class B>                                                      \
-        constexpr fst_t<B, R>                                                   \
-        simd_apply(const B& arg) const                                          \
+        constexpr auto simd_apply(const B& arg) const                           \
         {                                                                       \
             return OP arg;                                                      \
         }                                                                       \
-        template <class U>                                                      \
-        struct rebind                                                           \
-        {                                                                       \
-            using type = NAME<U>;                                               \
-        };                                                                      \
     }
 
 #define UNARY_OPERATOR_FUNCTOR(NAME, OP) UNARY_OPERATOR_FUNCTOR_IMPL(NAME, OP, T)
@@ -69,34 +54,18 @@ namespace xt
        argument is uint64_t.
     */
 #define BINARY_OPERATOR_FUNCTOR_IMPL(NAME, OP, R)                                \
-    template <class T>                                                           \
     struct NAME                                                                  \
     {                                                                            \
-        template <class U, class RT>                                             \
-        using fvt_t = xt::detail::functor_value_type_t<U, RT>;                   \
-        template <class U, class RT>                                             \
-        using fst_t = xt::detail::functor_batch_simd_type_t<U, RT>;              \
-        using first_argument_type = T;                                           \
-        using second_argument_type = T;                                          \
-        using result_type = fvt_t<T, R>;                                         \
-        using simd_value_type = xsimd::simd_type<T>;                             \
-        using simd_result_type = fst_t<simd_value_type, R>;                      \
         template <class T1, class T2>                                            \
-        constexpr result_type operator()(const T1& arg1, const T2& arg2) const   \
+        constexpr auto operator()(const T1& arg1, const T2& arg2) const          \
         {                                                                        \
             return (arg1 OP arg2);                                               \
         }                                                                        \
         template <class B>                                                       \
-        constexpr fst_t<B, R>                                                    \
-        simd_apply(const B& arg1, const B& arg2) const                           \
+        constexpr auto simd_apply(const B& arg1, const B& arg2) const            \
         {                                                                        \
             return (arg1 OP arg2);                                               \
         }                                                                        \
-        template <class U>                                                       \
-        struct rebind                                                            \
-        {                                                                        \
-            using type = NAME<U>;                                                \
-        };                                                                       \
     }
 
 #define BINARY_OPERATOR_FUNCTOR(NAME, OP) BINARY_OPERATOR_FUNCTOR_IMPL(NAME, OP, T)
@@ -128,18 +97,13 @@ namespace xt
         BINARY_BOOL_OPERATOR_FUNCTOR(equal_to, ==);
         BINARY_BOOL_OPERATOR_FUNCTOR(not_equal_to, !=);
 
-        template <class T>
         struct conditional_ternary
         {
-            using result_type = T;
-            using simd_value_type = xsimd::simd_type<T>;
-            using simd_bool_type = xsimd::simd_bool_type<T>;
-            using simd_result_type = simd_value_type;
-
             template <class B>
             using get_batch_bool = typename xsimd::simd_traits<typename xsimd::revert_simd_traits<B>::type>::bool_type;
 
-            constexpr result_type operator()(bool t1, const T& t2, const T& t3) const noexcept
+            template <class A1, class A2>
+            constexpr auto operator()(bool t1, const A1& t2, const A2& t3) const noexcept
             {
                 return t1 ? t2 : t3;
             }
@@ -150,24 +114,17 @@ namespace xt
             {
                 return xsimd::select(t1, t2, t3);
             }
-            template <class U>
-            struct rebind
-            {
-                using type = conditional_ternary<U>;
-            };
         };
 
         template <class R>
         struct cast
         {
-            template <class T>
             struct functor
             {
-                using argument_type = T;
                 using result_type = R;
-                using simd_value_type = xsimd::simd_type<T>;
-                using simd_result_type = xt::detail::functor_simd_type_t<T, R>;
-                constexpr result_type operator()(const T& arg) const
+
+                template <class A1>
+                constexpr result_type operator()(const A1& arg) const
                 {
                     return static_cast<R>(arg);
                 }
@@ -177,73 +134,38 @@ namespace xt
                 {
                     return static_cast<R>(arg);
                 }*/
-                template <class U>
-                struct rebind
-                {
-                    using type = functor<U>;
-                };
             };
         };
 
         template <class Tag, class F, class... E>
         struct select_xfunction_expression;
 
-        template <class F, class... E>
-        struct select_xfunction_expression<xscalar_expression_tag, F, E...>
-        {
-            using type = typename select_xfunction_expression<xtensor_expression_tag, F, E...>::type;
-        };
-
         template <class F, class... E>
         struct select_xfunction_expression<xtensor_expression_tag, F, E...>
         {
-            using type = xfunction<F, typename F::result_type, E...>;
+            using type = xfunction<F, E...>;
         };
 
         template <class F, class... E>
         struct select_xfunction_expression<xoptional_expression_tag, F, E...>
         {
-            using type = xoptional_function<F, typename F::result_type, E...>;
+            using type = xfunction<F, E...>;
         };
 
         template <class Tag, class F, class... E>
         using select_xfunction_expression_t = typename select_xfunction_expression<Tag, F, E...>::type;
 
-        template <class Tag, template <class...> class F, class... E>
-        struct build_functor_type;
-
-        template <template <class...> class F, class... E>
-        struct build_functor_type<xscalar_expression_tag, F, E...>
-        {
-            using type = typename build_functor_type<xtensor_expression_tag, F, E...>::type;
-        };
-
-        template <template <class...> class F, class... E>
-        struct build_functor_type<xtensor_expression_tag, F, E...>
-        {
-            using type = F<common_value_type_t<std::decay_t<E>...>>;
-        };
-
-        template <template <class...> class F, class... E>
-        struct build_functor_type<xoptional_expression_tag, F, E...>
-        {
-            using type = F<common_value_type_t<std::decay_t<E>...>>;
-        };
-
-        template <class Tag, template <class...> class F, class... E>
-        using build_functor_type_t = typename build_functor_type<Tag, F, E...>::type;
-
-        template <template <class...> class F, class... E>
+        template <class F, class... E>
         struct xfunction_type
         {
             using expression_tag = xexpression_tag_t<E...>;
-            using functor_type = build_functor_type_t<expression_tag, F, E...>;
+            using functor_type = F;
             using type = select_xfunction_expression_t<expression_tag,
-                functor_type,
-                const_xclosure_t<E>...>;
+                                                       functor_type,
+                                                       const_xclosure_t<E>...>;
         };
 
-        template <template <class...> class F, class... E>
+        template <class F, class... E>
         inline auto make_xfunction(E&&... e) noexcept
         {
             using function_type = xfunction_type<F, E...>;
@@ -256,7 +178,7 @@ namespace xt
         // Wrapping the xfunction type in the xfunction_type metafunction avoids this evaluation when
         // the condition is false, since it leads to a tricky bug preventing from using operator+ and operator-
         // on vector and arrays iterators.
-        template <template <class...> class F, class... E>
+        template <class F, class... E>
         using xfunction_type_t = typename std::enable_if_t<has_xexpression<std::decay_t<E>...>::value,
                                                            xfunction_type<F, E...>>::type;
     }
@@ -722,28 +644,14 @@ namespace xt
         return detail::make_xfunction<detail::conditional_ternary>(std::forward<E1>(e1), std::forward<E2>(e2), std::forward<E3>(e3));
     }
 
-    /**
-     * @ingroup logical_operators
-     * @brief return vector of indices where T is not zero
-     *
-     * @param arr input array
-     * @return vector of \a index_types where arr is not equal to zero
-     */
-    template <class T>
-    inline auto nonzero(const T& arr)
-        -> std::vector<xindex_type_t<typename T::shape_type>>
+    namespace detail
     {
-        auto shape = arr.shape();
-        using index_type = xindex_type_t<typename T::shape_type>;
-        using size_type = typename T::size_type;
-
-        auto idx = xtl::make_sequence<index_type>(arr.dimension(), 0);
-        std::vector<index_type> indices;
-
-        auto next_idx = [&shape](index_type& idx) {
-            for (size_type j = shape.size(); j > 0; --j)
+        template <class S, class I>
+        inline auto next_idx(const S& shape, I& idx)
+        {
+            for (std::size_t j = shape.size(); j > 0; --j)
             {
-                size_type i = j - 1;
+                std::size_t i = j - 1;
                 if (idx[i] >= shape[i] - 1)
                 {
                     idx[i] = 0;
@@ -755,20 +663,57 @@ namespace xt
                 }
             }
             // return empty index, happens at last iteration step, but remains unused
-            return index_type();
-        };
+            return I();
+        }
+    }
+
+    /**
+     * @ingroup logical_operators
+     * @brief return vector of indices where T is not zero
+     *
+     * @param arr input array
+     * @return vector of vectors, one for each dimension of arr, containing
+     * the indices of the non-zero elements in that dimension
+     */
+    template <class T>
+    inline auto nonzero(const T& arr)
+    {
+        auto shape = arr.shape();
+        using index_type = xindex_type_t<typename T::shape_type>;
+        using size_type = typename T::size_type;
+
+        auto idx = xtl::make_sequence<index_type>(arr.dimension(), 0);
+        std::vector<std::vector<size_type>> indices(arr.dimension());
 
         size_type total_size = compute_size(shape);
-        for (size_type i = 0; i < total_size; i++, next_idx(idx))
+        for (size_type i = 0; i < total_size; i++, detail::next_idx(shape, idx))
         {
             if (arr.element(std::begin(idx), std::end(idx)))
             {
-                indices.push_back(idx);
+                for (std::size_t n = 0; n < indices.size(); ++n)
+                {
+                    indices.at(n).push_back(idx[n]);
+                }
             }
         }
+
         return indices;
     }
 
+    /**
+     * @ingroup logical_operators
+     * @brief return indices that are non-zero in the flattened version of arr,
+     * equivalent to nonzero(ravel<layout_type>(arr))[0];
+     *
+     * @param arr input array
+     * @return indices that are non-zero in the flattened version of arr
+     */
+    template <layout_type L, class T>
+    inline auto flatnonzero(const T& arr)
+    {
+        return nonzero(ravel<L>(arr))[0];
+    }
+
     /**
      * @ingroup logical_operators
      * @brief return vector of indices where condition is true
@@ -779,11 +724,39 @@ namespace xt
      */
     template <class T>
     inline auto where(const T& condition)
-        -> std::vector<xindex_type_t<typename T::shape_type>>
     {
         return nonzero(condition);
     }
 
+    /**
+     * @ingroup logical_operators
+     * @brief return vector of indices where arr is not zero
+     *
+     * @param arr input array
+     * @return vector of index_types where arr is not equal to zero
+     */
+    template <class T>
+    inline auto argwhere(const T& arr)
+    {
+        auto shape = arr.shape();
+        using index_type = xindex_type_t<typename T::shape_type>;
+        using size_type = typename T::size_type;
+
+        auto idx = xtl::make_sequence<index_type>(arr.dimension(), 0);
+        std::vector<index_type> indices;
+
+        size_type total_size = compute_size(shape);
+        for (size_type i = 0; i < total_size; i++, detail::next_idx(shape, idx))
+        {
+            if (arr.element(std::begin(idx), std::end(idx)))
+            {
+                indices.push_back(idx);
+            }
+        }
+
+        return indices;
+    }
+
     /**
     * @ingroup logical_operators
     * @brief Any
@@ -835,9 +808,9 @@ namespace xt
 
     template <class R, class E>
     inline auto cast(E&& e) noexcept
-        -> detail::xfunction_type_t<detail::cast<R>::template functor, E>
+        -> detail::xfunction_type_t<typename detail::cast<R>::functor, E>
     {
-        return detail::make_xfunction<detail::cast<R>::template functor>(std::forward<E>(e));
+        return detail::make_xfunction<typename detail::cast<R>::functor>(std::forward<E>(e));
     }
 }
 
