@@ -23,11 +23,37 @@
 namespace xt
 {
 
+    /*********************
+     * xscalar extension *
+     *********************/
+
+    namespace extension
+    {
+        template <class Tag, class CT>
+        struct xscalar_base_impl;
+
+        template <class CT>
+        struct xscalar_base_impl<xtensor_expression_tag, CT>
+        {
+            using type = xtensor_empty_base;
+        };
+
+        template <class CT>
+        struct xscalar_base : xscalar_base_impl<get_expression_tag_t<std::decay_t<CT>>, CT>
+        {
+        };
+
+        template <class CT>
+        using xscalar_base_t = typename xscalar_base<CT>::type;
+    }
+
     /***********
      * xscalar *
      ***********/
 
     // xscalar is a cheap wrapper for a scalar value as an xexpression.
+    template <class CT>
+    class xscalar;
 
     template <bool is_const, class CT>
     class xscalar_stepper;
@@ -35,9 +61,6 @@ namespace xt
     template <bool is_const, class CT>
     class xdummy_iterator;
 
-    template <class CT>
-    class xscalar;
-
     template <class CT>
     struct xiterable_inner_types<xscalar<CT>>
     {
@@ -51,11 +74,15 @@ namespace xt
 #define DL XTENSOR_DEFAULT_LAYOUT
     template <class CT>
     class xscalar : public xexpression<xscalar<CT>>,
-                    private xiterable<xscalar<CT>>
+                    private xiterable<xscalar<CT>>,
+                    public extension::xscalar_base_t<CT>
     {
     public:
 
         using self_type = xscalar<CT>;
+        using xexpression_type = std::decay_t<CT>;
+        using extension_base = extension::xscalar_base_t<CT>;
+        using expression_tag = typename extension_base::expression_tag;
 
         using value_type = std::decay_t<CT>;
         using reference = value_type&;
@@ -69,7 +96,6 @@ namespace xt
         using iterable_base = xiterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = inner_shape_type;
-        using expression_tag = xscalar_expression_tag;
 
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
@@ -146,11 +172,14 @@ namespace xt
         template <class It>
         const_reference element(It, It) const noexcept;
 
+        xexpression_type& expression() noexcept;
+        const xexpression_type& expression() const noexcept;
+
         template <class S>
         bool broadcast_shape(S& shape, bool reuse_cache = false) const noexcept;
 
         template <class S>
-        bool is_trivial_broadcast(const S& strides) const noexcept;
+        bool has_linear_assign(const S& strides) const noexcept;
 
         template <layout_type L = DL>
         iterator begin() noexcept;
@@ -246,8 +275,10 @@ namespace xt
 
         template <class align, class simd = simd_value_type>
         void store_simd(size_type i, const simd& e);
-        template <class align, class simd = simd_value_type>
-        simd load_simd(size_type i) const;
+        template <class align, class requested_type = value_type,
+                  std::size_t N = xsimd::simd_traits<requested_type>::size>
+        xsimd::simd_return_type<value_type, requested_type>
+        load_simd(size_type i) const;
 
     private:
 
@@ -422,32 +453,32 @@ namespace xt
     {
     };
 
-    /*******************************
-     * trivial_begin / trivial_end *
-     *******************************/
+    /*****************************
+     * linear_begin / linear_end *
+     *****************************/
 
     namespace detail
     {
         template <class CT>
-        constexpr auto trivial_begin(xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
+        constexpr auto linear_begin(xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
         {
             return c.dummy_begin();
         }
 
         template <class CT>
-        constexpr auto trivial_end(xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
+        constexpr auto linear_end(xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
         {
             return c.dummy_end();
         }
 
         template <class CT>
-        constexpr auto trivial_begin(const xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
+        constexpr auto linear_begin(const xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
         {
             return c.dummy_begin();
         }
 
         template <class CT>
-        constexpr auto trivial_end(const xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
+        constexpr auto linear_end(const xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
         {
             return c.dummy_end();
         }
@@ -611,6 +642,18 @@ namespace xt
         return m_value;
     }
 
+    template <class CT>
+    inline auto xscalar<CT>::expression() noexcept -> xexpression_type&
+    {
+        return m_value;
+    }
+
+    template <class CT>
+    inline auto xscalar<CT>::expression() const noexcept -> const xexpression_type&
+    {
+        return m_value;
+    }
+
     template <class CT>
     template <class S>
     inline bool xscalar<CT>::broadcast_shape(S&, bool) const noexcept
@@ -620,7 +663,7 @@ namespace xt
 
     template <class CT>
     template <class S>
-    inline bool xscalar<CT>::is_trivial_broadcast(const S&) const noexcept
+    inline bool xscalar<CT>::has_linear_assign(const S&) const noexcept
     {
         return true;
     }
@@ -941,10 +984,11 @@ namespace xt
     }
 
     template <class CT>
-    template <class align, class simd>
-    inline auto xscalar<CT>::load_simd(size_type) const -> simd
+    template <class align, class requested_type, std::size_t N>
+    inline auto xscalar<CT>::load_simd(size_type) const
+        -> xsimd::simd_return_type<value_type, requested_type>
     {
-        return xsimd::set_simd<value_type, typename simd::value_type>(m_value);
+        return xsimd::set_simd<value_type, requested_type>(m_value);
     }
 
     template <class T>
