@@ -19,7 +19,6 @@
 #include "xstorage.hpp"
 #include "xutils.hpp"
 
-
 #ifndef XTENSOR_CONSTEXPR
     #if(defined(_MSC_VER) || __GNUC__ < 8)
         #define XTENSOR_CONSTEXPR inline
@@ -32,91 +31,6 @@
 
 namespace xt
 {
-    struct xall_tag {};
-    struct xnewaxis_tag {};
-    struct xellipsis_tag {};
-
-    template <class A, class B, class C>
-    struct xrange_adaptor;
-
-    namespace placeholders
-    {
-        // xtensor universal placeholder
-        struct xtuph {};
-
-        template <class... Args>
-        struct rangemaker
-        {
-            std::ptrdiff_t rng[3]; // = { 0, 0, 0 };
-        };
-
-        XTENSOR_CONSTEXPR xtuph get_tuph_or_val(std::ptrdiff_t /*val*/, std::true_type)
-        {
-            return xtuph();
-        }
-
-        XTENSOR_CONSTEXPR std::ptrdiff_t get_tuph_or_val(std::ptrdiff_t val, std::false_type)
-        {
-            return val;
-        }
-
-        template <class A, class B, class C>
-        struct rangemaker<A, B, C>
-        {
-            XTENSOR_CONSTEXPR operator xrange_adaptor<A, B, C>()
-            {
-                return xrange_adaptor<A, B, C>({
-                    get_tuph_or_val(rng[0], std::is_same<A, xtuph>()),
-                    get_tuph_or_val(rng[1], std::is_same<B, xtuph>()),
-                    get_tuph_or_val(rng[2], std::is_same<C, xtuph>())
-                });
-            }
-
-            std::ptrdiff_t rng[3];// = { 0, 0, 0 };
-        };
-
-        template <class A, class B>
-        struct rangemaker<A, B>
-        {
-            XTENSOR_CONSTEXPR operator xrange_adaptor<A, B, xt::placeholders::xtuph>()
-            {
-                return xrange_adaptor<A, B, xt::placeholders::xtuph>({
-                    get_tuph_or_val(rng[0], std::is_same<A, xtuph>()),
-                    get_tuph_or_val(rng[1], std::is_same<B, xtuph>()),
-                    xtuph()
-                });
-            }
-
-            std::ptrdiff_t rng[3];  // = { 0, 0, 0 };
-        };
-
-        template <class... OA>
-        XTENSOR_CONSTEXPR auto operator|(const rangemaker<OA...>& rng, const std::ptrdiff_t& t)
-        {
-            auto nrng = rangemaker<OA..., std::ptrdiff_t>({rng.rng[0], rng.rng[1], rng.rng[2]});
-            nrng.rng[sizeof...(OA)] = t;
-            return nrng;
-        }
-
-        template <class... OA>
-        XTENSOR_CONSTEXPR auto operator|(const rangemaker<OA...>& rng, const xt::placeholders::xtuph& /*t*/)
-        {
-            auto nrng = rangemaker<OA..., xt::placeholders::xtuph>({rng.rng[0], rng.rng[1], rng.rng[2]});
-            return nrng;
-        }
-
-
-        XTENSOR_GLOBAL_CONSTEXPR xtuph _{};
-        XTENSOR_GLOBAL_CONSTEXPR rangemaker<> _r = rangemaker<>({0, 0, 0});
-        XTENSOR_GLOBAL_CONSTEXPR xall_tag _a{};
-        XTENSOR_GLOBAL_CONSTEXPR xnewaxis_tag _n{};
-        XTENSOR_GLOBAL_CONSTEXPR xellipsis_tag _e{};
-    }
-
-    inline auto xnone()
-    {
-        return placeholders::xtuph();
-    }
 
     /**********************
      * xslice declaration *
@@ -153,6 +67,23 @@ namespace xt
     template <class... E>
     using has_xslice = xtl::disjunction<is_xslice<E>...>;
 
+    /**************
+     * slice tags *
+     **************/
+
+#define DEFINE_TAG_CONVERSION(NAME)                 \
+    template <class T>                              \
+    XTENSOR_CONSTEXPR NAME convert() const noexcept \
+    {                                               \
+        return NAME();                              \
+    }
+
+    struct xall_tag { DEFINE_TAG_CONVERSION(xall_tag) };
+    struct xnewaxis_tag { DEFINE_TAG_CONVERSION(xnewaxis_tag) };
+    struct xellipsis_tag { DEFINE_TAG_CONVERSION(xellipsis_tag) };
+
+#undef DEFINE_TAG_CONVERSION
+
     /**********************
      * xrange declaration *
      **********************/
@@ -171,6 +102,11 @@ namespace xt
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xrange<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xrange<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
 
         size_type size() const noexcept;
@@ -210,6 +146,11 @@ namespace xt
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xstepped_range<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xstepped_range<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
 
         size_type size() const noexcept;
@@ -250,6 +191,11 @@ namespace xt
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xall<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xall<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
 
         size_type size() const noexcept;
@@ -308,12 +254,18 @@ namespace xt
     public:
 
         using size_type = T;
+        using self_type = xnewaxis<T>;
 
         xnewaxis() = default;
 
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xnewaxis<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xnewaxis<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
 
         size_type size() const noexcept;
@@ -322,6 +274,9 @@ namespace xt
         size_type revert_index(std::size_t i) const noexcept;
 
         bool contains(size_type i) const noexcept;
+
+        bool operator==(const self_type& rhs) const noexcept;
+        bool operator!=(const self_type& rhs) const noexcept;
     };
 
     /**
@@ -379,6 +334,11 @@ namespace xt
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xkeep_slice<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xkeep_slice<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
         size_type size() const noexcept;
 
@@ -496,6 +456,11 @@ namespace xt
         template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
         operator xdrop_slice<S>() const noexcept;
 
+        // Same as implicit conversion operator but more convenient to call
+        // from a variant visitor
+        template <class S, typename = std::enable_if_t<std::is_convertible<S, T>::value, void>>
+        xdrop_slice<S> convert() const noexcept;
+
         size_type operator()(size_type i) const noexcept;
         size_type size() const noexcept;
 
@@ -569,11 +534,11 @@ namespace xt
         return slice_type(std::move(tmp));
     }
 
-    /******************
-     * xrange_adaptor *
-     ******************/
+    /******************************
+     * xrange_adaptor declaration *
+     ******************************/
 
-    template <class A, class B, class C>
+    template <class A, class B = A, class C = A>
     struct xrange_adaptor
     {
         xrange_adaptor(A start_val, B stop_val, C step)
@@ -698,6 +663,89 @@ namespace xt
         C m_step;
     };
 
+    /*******************************
+     * Placeholders and rangemaker *
+     *******************************/
+
+    namespace placeholders
+    {
+        // xtensor universal placeholder
+        struct xtuph {};
+
+        template <class... Args>
+        struct rangemaker
+        {
+            std::ptrdiff_t rng[3]; // = { 0, 0, 0 };
+        };
+
+        XTENSOR_CONSTEXPR xtuph get_tuph_or_val(std::ptrdiff_t /*val*/, std::true_type)
+        {
+            return xtuph();
+        }
+
+        XTENSOR_CONSTEXPR std::ptrdiff_t get_tuph_or_val(std::ptrdiff_t val, std::false_type)
+        {
+            return val;
+        }
+
+        template <class A, class B, class C>
+        struct rangemaker<A, B, C>
+        {
+            XTENSOR_CONSTEXPR operator xrange_adaptor<A, B, C>()
+            {
+                return xrange_adaptor<A, B, C>({
+                    get_tuph_or_val(rng[0], std::is_same<A, xtuph>()),
+                    get_tuph_or_val(rng[1], std::is_same<B, xtuph>()),
+                    get_tuph_or_val(rng[2], std::is_same<C, xtuph>())
+                });
+            }
+
+            std::ptrdiff_t rng[3];// = { 0, 0, 0 };
+        };
+
+        template <class A, class B>
+        struct rangemaker<A, B>
+        {
+            XTENSOR_CONSTEXPR operator xrange_adaptor<A, B, xt::placeholders::xtuph>()
+            {
+                return xrange_adaptor<A, B, xt::placeholders::xtuph>({
+                    get_tuph_or_val(rng[0], std::is_same<A, xtuph>()),
+                    get_tuph_or_val(rng[1], std::is_same<B, xtuph>()),
+                    xtuph()
+                });
+            }
+
+            std::ptrdiff_t rng[3];  // = { 0, 0, 0 };
+        };
+
+        template <class... OA>
+        XTENSOR_CONSTEXPR auto operator|(const rangemaker<OA...>& rng, const std::ptrdiff_t& t)
+        {
+            auto nrng = rangemaker<OA..., std::ptrdiff_t>({ rng.rng[0], rng.rng[1], rng.rng[2] });
+            nrng.rng[sizeof...(OA)] = t;
+            return nrng;
+        }
+
+        template <class... OA>
+        XTENSOR_CONSTEXPR auto operator|(const rangemaker<OA...>& rng, const xt::placeholders::xtuph& /*t*/)
+        {
+            auto nrng = rangemaker<OA..., xt::placeholders::xtuph>({ rng.rng[0], rng.rng[1], rng.rng[2] });
+            return nrng;
+        }
+
+
+        XTENSOR_GLOBAL_CONSTEXPR xtuph _{};
+        XTENSOR_GLOBAL_CONSTEXPR rangemaker<> _r = rangemaker<>({ 0, 0, 0 });
+        XTENSOR_GLOBAL_CONSTEXPR xall_tag _a{};
+        XTENSOR_GLOBAL_CONSTEXPR xnewaxis_tag _n{};
+        XTENSOR_GLOBAL_CONSTEXPR xellipsis_tag _e{};
+    }
+
+    inline auto xnone()
+    {
+        return placeholders::xtuph();
+    }
+
     namespace detail
     {
         template <class T, class E = void>
@@ -765,7 +813,6 @@ namespace xt
             detail::cast_if_integer<A>{}(start_val), detail::cast_if_integer<B>{}(stop_val), detail::cast_if_integer<C>{}(step));
     }
 
-
     /******************************************************
      * homogeneous get_size for integral types and slices *
      ******************************************************/
@@ -844,6 +891,13 @@ namespace xt
         return slice;
     }
 
+    template <class E, class T>
+    inline auto get_slice_implementation(E& e, xdrop_slice<T>&& slice, std::size_t index)
+    {
+        slice.normalize(e.shape()[index]);
+        return slice;
+    }
+
     template <class E>
     inline auto get_slice_implementation(E& e, xall_tag&&, std::size_t index)
     {
@@ -932,6 +986,13 @@ namespace xt
         return ret;
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xrange<S> xrange<T>::convert() const noexcept
+    {
+        return xrange<S>(*this);
+    }
+
     template <class T>
     inline auto xrange<T>::operator()(size_type i) const noexcept -> size_type
     {
@@ -986,8 +1047,10 @@ namespace xt
 
     template <class T>
     inline xstepped_range<T>::xstepped_range(size_type start_val, size_type stop_val, size_type step) noexcept
-        : m_start(start_val), m_size(size_type(std::ceil(double(stop_val - start_val) / double(step)))), m_step(step)
+        : m_start(start_val), m_size(size_type(0)), m_step(step)
     {
+        size_type n = stop_val - start_val;
+        m_size = n / step + (((n < 0) ^ (step > 0)) && (n % step));
     }
 
     template <class T>
@@ -1001,6 +1064,13 @@ namespace xt
         return ret;
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xstepped_range<S> xstepped_range<T>::convert() const noexcept
+    {
+        return xstepped_range<S>(*this);
+    }
+
     template <class T>
     inline auto xstepped_range<T>::operator()(size_type i) const noexcept -> size_type
     {
@@ -1066,6 +1136,13 @@ namespace xt
         return xall<S>(static_cast<S>(m_size));
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xall<S> xall<T>::convert() const noexcept
+    {
+        return xall<S>(*this);
+    }
+
     template <class T>
     inline auto xall<T>::operator()(size_type i) const noexcept -> size_type
     {
@@ -1125,6 +1202,13 @@ namespace xt
         return xnewaxis<S>();
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xnewaxis<S> xnewaxis<T>::convert() const noexcept
+    {
+        return xnewaxis<S>(*this);
+    }
+
     template <class T>
     inline auto xnewaxis<T>::operator()(size_type) const noexcept -> size_type
     {
@@ -1161,6 +1245,18 @@ namespace xt
         return i == 0;
     }
 
+    template <class T>
+    inline bool xnewaxis<T>::operator==(const self_type& /*rhs*/) const noexcept
+    {
+        return true;
+    }
+
+    template <class T>
+    inline bool xnewaxis<T>::operator!=(const self_type& /*rhs*/) const noexcept
+    {
+        return true;
+    }
+
     /******************************
      * xkeep_slice implementation *
      ******************************/
@@ -1192,8 +1288,10 @@ namespace xt
     inline xkeep_slice<T>::operator xkeep_slice<S>() const noexcept
     {
         xkeep_slice<S> ret;
-        ret.m_raw_indices.resize(size());
-        ret.m_indices.resize(size());
+        using us_type = typename container_type::size_type;
+        us_type sz = static_cast<us_type>(size());
+        ret.m_raw_indices.resize(sz);
+        ret.m_indices.resize(sz);
         std::transform(m_raw_indices.cbegin(), m_raw_indices.cend(), ret.m_raw_indices.begin(),
                        [](const T& val) { return static_cast<S>(val); });
         std::transform(m_indices.cbegin(), m_indices.cend(), ret.m_indices.begin(),
@@ -1201,6 +1299,13 @@ namespace xt
         return ret;
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xkeep_slice<S> xkeep_slice<T>::convert() const noexcept
+    {
+        return xkeep_slice<S>(*this);
+    }
+
     template <class T>
     inline void xkeep_slice<T>::normalize(std::size_t shape)
     {
@@ -1227,14 +1332,12 @@ namespace xt
     template <class T>
     inline auto xkeep_slice<T>::step_size(std::size_t i, std::size_t n) const noexcept -> size_type
     {
-        // special case one-past-end step (should be removed soon)
-        if (i == m_indices.size())
+        if (i + 1 == m_indices.size())
         {
             return 1;
         }
         else
         {
-            --i;
             return m_indices[i + n] - m_indices[i];
         }
     }
@@ -1314,6 +1417,13 @@ namespace xt
         return ret;
     }
 
+    template <class T>
+    template <class S, typename>
+    inline xdrop_slice<S> xdrop_slice<T>::convert() const noexcept
+    {
+        return xdrop_slice<S>(*this);
+    }
+
     template <class T>
     inline void xdrop_slice<T>::normalize(std::size_t shape)
     {
@@ -1364,8 +1474,7 @@ namespace xt
     template <class T>
     inline auto xdrop_slice<T>::step_size(std::size_t i, std::size_t n) const noexcept -> size_type
     {
-        // special case one-past-end step (should be removed soon)
-        if (i == static_cast<size_type>(m_indices.size()))
+        if(i + 1 == static_cast<std::size_t>(m_size))
         {
             return 1;
         }
