@@ -34,6 +34,30 @@
 namespace xt
 {
 
+    /*******************
+     * xview extension *
+     *******************/
+
+    namespace extension
+    {
+        template <class Tag, class CT, class... S>
+        struct xview_base_impl;
+
+        template <class CT, class... S>
+        struct xview_base_impl<xtensor_expression_tag, CT, S...>
+        {
+            using type = xtensor_empty_base;
+        };
+
+        template <class CT, class... S>
+        struct xview_base : xview_base_impl<xexpression_tag_t<CT>, CT, S...>
+        {
+        };
+
+        template <class CT, class... S>
+        using xview_base_t = typename xview_base<CT, S...>::type;
+    }
+
     /*********************
      * xview declaration *
      *********************/
@@ -305,7 +329,8 @@ namespace xt
      */
     template <class CT, class... S>
     class xview : public xview_semantic<xview<CT, S...>>,
-                  public xiterable<xview<CT, S...>>
+                  public xiterable<xview<CT, S...>>,
+                  public extension::xview_base_t<CT, S...>
     {
     public:
 
@@ -314,6 +339,9 @@ namespace xt
         using semantic_base = xview_semantic<self_type>;
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
 
+        using extension_base = extension::xview_base_t<CT, S...>;
+        using expression_tag = typename extension_base::expression_tag;
+
         static constexpr bool is_const = std::is_const<std::remove_reference_t<CT>>::value;
         using value_type = typename xexpression_type::value_type;
         using simd_value_type = xsimd::simd_type<value_type>;
@@ -424,11 +452,14 @@ namespace xt
         template <class It>
         const_reference element(It first, It last) const;
 
+        xexpression_type& expression() noexcept;
+        const xexpression_type& expression() const noexcept;
+
         template <class ST>
         bool broadcast_shape(ST& shape, bool reuse_cache = false) const;
 
         template <class ST>
-        bool is_trivial_broadcast(const ST& strides) const;
+        bool has_linear_assign(const ST& strides) const;
 
         template <class ST, bool Enable = is_strided_view>
         std::enable_if_t<!Enable, stepper>
@@ -536,21 +567,29 @@ namespace xt
                   class = std::enable_if_t<has_data_interface<T>::value && is_contiguous_view, int>>
         void assign_to(xexpression<E>& e, bool force_resize) const;
 
+        template <class E>
+        using rebind_t = xview<E, S...>;
+
+        template <class E>
+        rebind_t<E> build_view(E&& e) const;
+
         //
         // SIMD interface
         //
 
-        template <class simd>
-        using simd_return_type = xsimd::simd_return_type<value_type, typename simd::value_type>;
+        template <class requested_type>
+        using simd_return_type = xsimd::simd_return_type<value_type, requested_type>;
 
         template <class T, class R>
         using enable_simd_interface = std::enable_if_t<has_simd_interface<T>::value && is_strided_view, R>;
 
-        template <class align, class simd = simd_value_type, class T = xexpression_type>
+        template <class align, class simd, class T = xexpression_type>
         enable_simd_interface<T, void> store_simd(size_type i, const simd& e);
 
-        template <class align, class simd = simd_value_type, class T = xexpression_type>
-        enable_simd_interface<T, simd_return_type<simd>> load_simd(size_type i) const;
+        template <class align, class requested_type = value_type,
+                  std::size_t N = xsimd::simd_traits<requested_type>::size,
+                  class T = xexpression_type>
+        enable_simd_interface<T, simd_return_type<requested_type>> load_simd(size_type i) const;
 
         template <class T = xexpression_type>
         enable_simd_interface<T, reference> data_element(size_type i);
@@ -632,29 +671,16 @@ namespace xt
         void assign_temporary_impl(temporary_type&& tmp);
 
         template <std::size_t... I>
-        inline std::size_t data_offset_impl(std::index_sequence<I...>) const noexcept;
+        std::size_t data_offset_impl(std::index_sequence<I...>) const noexcept;
 
         template <std::size_t... I>
-        inline auto compute_strides_impl(std::index_sequence<I...>) const noexcept;
+        auto compute_strides_impl(std::index_sequence<I...>) const noexcept;
 
-        auto compute_shape(std::true_type) const
-        {
-            return inner_shape_type(m_e.shape());
-        }
+        inner_shape_type compute_shape(std::true_type) const;
+        inner_shape_type compute_shape(std::false_type) const;
 
-        auto compute_shape(std::false_type) const
-        {
-            std::size_t dim = m_e.dimension() - integral_count<S...>() + newaxis_count<S...>();
-            auto shape = xtl::make_sequence<inner_shape_type>(dim, 0);
-            auto func = [](const auto& s) noexcept { return get_size(s); };
-            for (size_type i = 0; i != dim; ++i)
-            {
-                size_type index = integral_skip<S...>(i);
-                shape[i] = index < sizeof...(S) ?
-                    apply<size_type>(index, func, m_slices) : m_e.shape()[index - newaxis_count_before<S...>(index)];
-            }
-            return shape;
-        }
+        template <class E, std::size_t... I>
+        rebind_t<E> build_view_impl(E&& e, std::index_sequence<I...>) const;
 
         friend class xview_semantic<xview<CT, S...>>;
     };
@@ -1131,6 +1157,24 @@ namespace xt
         return m_e.element(index.cbegin(), index.cend());
     }
 
+    /**
+     * Returns a reference to the underlying expression of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::expression() noexcept -> xexpression_type&
+    {
+        return m_e;
+    }
+
+    /**
+     * Returns a const reference to the underlying expression of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::expression() const noexcept -> const xexpression_type&
+    {
+        return m_e;
+    }
+
     /**
      * Returns the data holder of the underlying container (only if the view is on a realized
      * container). ``xt::eval`` will make sure that the underlying xexpression is
@@ -1306,13 +1350,13 @@ namespace xt
     }
 
     /**
-     * Compares the specified strides with those of the view to see whether
-     * the broadcasting is trivial.
-     * @return a boolean indicating whether the broadcasting is trivial
+     * Checks whether the xview can be linearly assigned to an expression
+     * with the specified strides.
+     * @return a boolean indicating whether a linear assign is possible
      */
     template <class CT, class... S>
     template <class ST>
-    inline bool xview<CT, S...>::is_trivial_broadcast(const ST& str) const
+    inline bool xview<CT, S...>::has_linear_assign(const ST& str) const
     {
         return xtl::mpl::static_if<is_strided_view>([&](auto self)
         {
@@ -1374,6 +1418,20 @@ namespace xt
         std::copy(data() + data_offset(), data() + data_offset() + de.size(), de.template begin<static_layout>());
     }
 
+    template <class CT, class... S>
+    template <class E, std::size_t... I>
+    inline auto xview<CT, S...>::build_view_impl(E&& e, std::index_sequence<I...>) const -> rebind_t<E>
+    {
+        return rebind_t<E>(std::forward<E>(e), std::get<I>(m_slices)...);
+    }
+
+    template <class CT, class... S>
+    template <class E>
+    inline auto xview<CT, S...>::build_view(E&& e) const -> rebind_t<E>
+    {
+        return build_view_impl(std::forward<E>(e), std::make_index_sequence<sizeof...(S)>());
+    }
+
     template <class CT, class... S>
     template <class align, class simd, class T>
     inline auto xview<CT, S...>::store_simd(size_type i, const simd& e) -> enable_simd_interface<T, void>
@@ -1382,10 +1440,10 @@ namespace xt
     }
 
     template <class CT, class... S>
-    template <class align, class simd, class T>
-    inline auto xview<CT, S...>::load_simd(size_type i) const -> enable_simd_interface<T, simd_return_type<simd>>
+    template <class align, class requested_type, std::size_t N, class T>
+    inline auto xview<CT, S...>::load_simd(size_type i) const -> enable_simd_interface<T, simd_return_type<requested_type>>
     {
-        return m_e.template load_simd<xsimd::unaligned_mode, simd>(data_offset() + i);
+        return m_e.template load_simd<xsimd::unaligned_mode, requested_type>(data_offset() + i);
     }
 
     template <class CT, class... S>
@@ -1578,12 +1636,33 @@ namespace xt
         return index;
     }
 
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::compute_shape(std::true_type) const -> inner_shape_type
+    {
+        return inner_shape_type(m_e.shape());
+    }
+
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::compute_shape(std::false_type) const -> inner_shape_type
+    {
+        std::size_t dim = m_e.dimension() - integral_count<S...>() + newaxis_count<S...>();
+        auto shape = xtl::make_sequence<inner_shape_type>(dim, 0);
+        auto func = [](const auto& s) noexcept { return get_size(s); };
+        for (size_type i = 0; i != dim; ++i)
+        {
+            size_type index = integral_skip<S...>(i);
+            shape[i] = index < sizeof...(S) ?
+                apply<size_type>(index, func, m_slices) : m_e.shape()[index - newaxis_count_before<S...>(index)];
+        }
+        return shape;
+    }
+
     namespace xview_detail
     {
         template <class V, class T>
         inline void run_assign_temporary_impl(V& v, const T& t, std::true_type /* enable strided assign */)
         {
-            strided_assign(v, t, std::true_type{});
+            strided_loop_assigner<true>::run(v, t);
         }
 
         template <class V, class T>
@@ -1880,7 +1959,11 @@ namespace xt
         {
             auto func = [&dim, this](const auto& s) noexcept {
                 this->m_index_keeper[dim]++;
-                return step_size(s, this->m_index_keeper[dim], 1);
+                // index_keeper tracks index+1, this should be refactored
+                // there are some pitfalls in to_end_impl and common_reset
+                // methods
+                auto idx = this->m_index_keeper[dim] - 1;
+                return step_size(s, idx, 1);
             };
             size_type index = integral_skip<S...>(dim);
             if (!is_newaxis_slice(index))
@@ -1901,7 +1984,8 @@ namespace xt
         {
             auto func = [&dim, &n, this](const auto& s) noexcept {
                 this->m_index_keeper[dim] += n;
-                return step_size(s, this->m_index_keeper[dim], n);
+                auto idx = this->m_index_keeper[dim] - 1;
+                return step_size(s, idx, n);
             };
 
             size_type index = integral_skip<S...>(dim);
@@ -1923,7 +2007,11 @@ namespace xt
         {
             auto func = [&dim, this](const auto& s) noexcept {
                 this->m_index_keeper[dim]--;
-                return step_size(s, this->m_index_keeper[dim], 1);
+                // index_keeper tracks index+1, this should be refactored
+                // there are some pitfalls in to_end_impl and common_reset
+                // methods
+                auto idx = this->m_index_keeper[dim] - 1;
+                return step_size(s, idx, 1);
             };
             size_type index = integral_skip<S...>(dim);
             if (!is_newaxis_slice(index))
@@ -1944,7 +2032,11 @@ namespace xt
         {
             auto func = [&dim, &n, this](const auto& s) noexcept {
                 this->m_index_keeper[dim] -= n;
-                return step_size(s, this->m_index_keeper[dim], n);
+                // index_keeper tracks index+1, this should be refactored
+                // there are some pitfalls in to_end_impl and common_reset
+                // methods
+                auto idx = this->m_index_keeper[dim] - 1;
+                return step_size(s, idx, n);
             };
 
             size_type index = integral_skip<S...>(dim);
